// -*- C++ -*-

// Interface for JGFloat64 type. See JGByte.h
//
// jhrg 1/12/95

#ifndef _jgfloat64_h
#define _jgfloat64_h 1

#ifndef __POWERPC__
#ifdef __GNUG__
#pragma interface
#endif
#endif

#include "Float64.h"

class JGFloat64: public Float64 {
private:
    dods_float64 _missing_val;

public:
    JGFloat64(const string &n = (char *)0);
    virtual ~JGFloat64() {}

    virtual BaseType *ptr_duplicate();
    
    virtual bool read(const string &dataset);

    virtual dods_float64 get_missing_value() const;
    virtual void set_missing_value(const dods_float64 &v);
    virtual void set_missing_value(const  string &attribute);
};

/* 
 * $Log: JGFloat64.h,v $
 * Revision 1.8.4.1  2003/06/26 09:22:14  rmorris
 * ifdef'd out usage of #pragma interface for OS_X.  That's not compatible
 * with dynamic typing under OS_X.
 *
 * Revision 1.8  2003/01/16 07:57:34  jimg
 * Merged changes from release-3-2-6.
 * Removed a number of files used for the client library ans sample clients.
 *
 * Revision 1.6.4.2  2003/01/14 05:36:46  jimg
 * I added a set_missing_value(const string &) method. This lets the code
 * in jgdds.cc that builds a DDS (which may contain types other than string
 * if that has been configured in the dataset's DAS) set up the missing
 * values using information from the DAS without doing string to value
 * conversions for each numeric type in a big switch statement.
 *
 * Revision 1.7  2001/08/27 17:41:03  jimg
 * Merged with release-3-2-2
 *
 * Revision 1.6.4.1  2001/08/21 15:27:21  dan
 * Updated names of the set/get accessor methods for modifying the missing_value
 * member field, to make the naming convention consistent with the current
 * DAP usage.
 *
 * Revision 1.6  2000/10/10 23:36:14  jimg
 * Added exception processing code.
 * Added throws to the read methods.
 * Changed the definition of read to match the dap library.
 * Moved CVS Log entries to the end of files.
 *
 * Revision 1.5  1999/11/05 01:17:23  jimg
 * Result of merge with release-3-1-0
 *
 * Revision 1.4.6.1  1999/10/24 18:31:50  dan
 * Modified to support type conversion from JGOFS
 * string variables to dods_float64.
 *
 * Revision 1.4  1999/05/05 04:38:00  brent
 * convert String.h code to new stanard lib <string>
 *
 * Revision 1.3  1996/09/17 00:02:10  jimg
 * Misc changes associated with merging branch code to the trunk.
 *
 * Revision 1.2.8.1.2.1  1996/06/26 22:17:49  jimg
 * Dans changes for 2.05.
 *
 * Revision 1.2  1995/03/14  01:23:36  jimg
 * Modified so that these files are a proper subclass of the DAP classes
 * Byte, ..., Grid.
 * Added assert macros.
 * Added Jgofs.h external declarations file.
 *
 * Revision 1.1  1995/02/16  18:59:59  dan
 * JGOFS sub-class definitions for Float64.h
 *
 * Revision 1.2  1995/01/19  21:59:31  jimg
 * Added read_val from dummy_read.cc to the sample set of sub-class
 * implementations.
 * Changed the declaration of readVal in BaseType so that it names the
 * mfunc read_val (to be consistant with the other mfunc names).
 * Removed the unnecessary duplicate declaration of the abstract virtual
 * mfuncs read and (now) read_val from the classes Byte, ... Grid. The
 * declaration in BaseType is sufficient along with the decl and definition
 * in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
 *
 * Revision 1.1  1995/01/19  20:20:40  jimg
 * Created as an example of subclassing the class hierarchy rooted at
 * BaseType.
 *
 */

#endif

