
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      dan		Dan Holloway (dholloway@gso.uri.edu)

// Implementation for the class JGSequence. See JGByte.cc
//
// jhrg 1/12/95

#include "config_jg.h"

#ifdef _GNUG_
#pragma implementation
#endif

#include "JGSequence.h"
#include "jgofs.h"

JGSequence *
NewSequence(const string &n)
{
    return new JGSequence(n);
}

BaseType *
JGSequence::ptr_duplicate()
{
    return new JGSequence(*this);
}

JGSequence::JGSequence(const string &n) : Sequence(n) //, _use_buffer(false)
{
}

JGSequence::~JGSequence()
{
}

bool
JGSequence::read(const string &dataset)
{
    bool status = true;
    int level;

    // Warning from g++: varValue is a char (*)[80] but the formal param is
    // an fourty element array.

    // I added the following variable and modified the if statement below
    // since varValue is an array of things that are 80 bytes long. However,
    // varSize is set to 40 in jgdds.cc so ljdbreada_() stuffs values in it
    // every 40 chars. When other code goes to read values out it uses the
    // decalred size of the array (80) and thus skips every other value. 
    // Note that the symbol VALSIZE is defined to be `80'.
    // jhrg 11/7/96

    int valSize = VALSIZE;

    if ((level = ljdbreada_(&unit, (char *)varValue, 
			    &valSize)) >= 0) {

	for (Pix p = first_var(); p; next_var(p)) {
	    if (var(p)->type() == dods_sequence_c) {
		JGSequence *jgs = dynamic_cast<JGSequence *>(var(p));
		status = read_level(dataset, jgs);
		var(p)->set_read_p(status);
	    }	
	    else {
		// Note: read will throw an exception if it encounters an
		// error. 
		status = var(p)->read(dataset);
	    }
	}
    }
    else {
	status = false;
    }

    return status;
}

// Make read_level use the same semantics as read; return false for no more
// data, true for more data and set `error' if/when a problem is found.

bool
JGSequence::read_level(const string &dataset, JGSequence *seq)
{
    bool status = true;

    for (Pix p = seq->first_var(); p; seq->next_var(p)) {
	if (seq->var(p)->type() == dods_sequence_c) {
	    JGSequence *jgs = dynamic_cast<JGSequence *>(seq->var(p));
	    status = read_level(dataset, jgs);
	    seq->set_read_p(status);
	}
	else { 
	  status = seq->var(p)->read(dataset);
	}
    }

    return status;
}

// $Log: JGSequence.cc,v $
// Revision 1.16  2003/01/16 23:21:37  jimg
// Removed old code and reset DODS_DEBUG. These changes were made on the
// release-3-2 branch but didn't get included in the code tagged
// release-3-2-5, so they were not merged here. Since they were so simple,
// I just made them by hand here.
//
// Revision 1.15  2003/01/16 07:57:34  jimg
// Merged changes from release-3-2-6.
// Removed a number of files used for the client library and sample clients.
//
// Revision 1.13.4.3  2003/01/16 01:57:27  jimg
// Added a cast for varValue.
//
// Revision 1.13.4.2  2002/06/03 22:23:37  jimg
// Removed calls to the Sequence::level and set_level methods. These were not
// actually used anywhere; only values were set or printed out by
// instrumentation. Since I removed these methods from the C++ code, I have to
// remove them from this code as well.
//
// Revision 1.14  2001/09/28 23:36:13  jimg
// Merged with 3.2.3.
//
// Revision 1.13.4.1  2001/09/26 23:34:48  jimg
// Replaced old-style cast with dynamic_cast.
//
// Revision 1.13  2000/10/10 23:36:14  jimg
// Added exception processing code.
// Added throws to the read methods.
// Changed the definition of read to match the dap library.
// Moved CVS Log entries to the end of files.
//
// Revision 1.12  1999/05/05 04:38:00  brent
// convert String.h code to new stanard lib <string>
//
// Revision 1.11  1998/08/06 16:32:35  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.10  1998/02/19 19:47:18  jimg
// Reverted to old read() member function.
// Removed stuff for the optimized Sequence transfer.
//
// Revision 1.9  1998/02/05 20:14:34  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.8  1997/10/09 22:19:42  jimg
// Resolved conflicts in merge of 2.14c to trunk.
//
// Revision 1.7  1996/11/08 01:01:44  jimg
// Fixed a problem in read() where values were packed to tightly (every 40
// bytes rather than 80). This caused other code to effectively skip every
// other data value.
//
// Revision 1.6  1996/09/17 00:02:01  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.5.2.1.2.1  1996/06/26 22:18:02  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:12  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3  1995/07/09  21:39:53  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  01:23:31  jimg
// Modified so that these files are a proper subclass of the DAP classes
// Byte, ..., Grid.
// Added assert macros.
// Added Jgofs.h external declarations file.
//
// Revision 1.1  1995/02/16  19:02:31  dan
// JGOFS sub-class definitions for Sequence.cc
//
// Revision 1.2  1995/01/19  21:59:00  jimg
// Added read_val from dummy_read.cc to the sample set of sub-class
// implementations.
// Changed the declaration of readVal in BaseType so that it names the
// mfunc read_val (to be consistant with the other mfunc names).
// Removed the unnecessary duplicate declaration of the abstract virtual
// mfuncs read and (now) read_val from the classes Byte, ... Grid. The
// declaration in BaseType is sufficient along with the decl and definition
// in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
//
// Revision 1.1  1995/01/19  20:20:51  jimg
// Created as an example of subclassing the class hierarchy rooted at
// BaseType.
//
